@@ -10,16 +10,19 @@ use common::{
 
 use odbc_api::{
     buffers::{
-        AnyColumnView, AnyColumnViewMut, BufferDescription, BufferKind, ColumnarRowSet, Indicator,
-        Item, TextRowSet,
+        buffer_from_description, buffer_from_description_and_indices, AnyColumnView,
+        AnyColumnViewMut, BufferDescription, BufferKind, ColumnarBuffer, Indicator, Item,
+        TextColumn, TextRowSet,
     },
     handles::{OutputStringBuffer, Statement},
-    parameter::{Blob, BlobRead, BlobSlice, VarBinaryArray, VarCharArray, VarCharSlice},
-    sys, ColumnDescription, Cursor, CursorImpl, DataType, InputParameter, IntoParameter,
-    Nullability, Nullable, Out, ResultSetMetadata, U16String,
+    parameter::InputParameter,
+    parameter::{
+        Blob, BlobRead, BlobSlice, VarBinaryArray, VarCharArray, VarCharSlice, WithDataType,
+    },
+    sys, Bit, ColumnDescription, Cursor, DataType, InOut, IntoParameter, Nullability, Nullable,
+    Out, ResultSetMetadata, U16String,
 };
 use std::{
-    convert::TryInto,
     ffi::CString,
     io::{self, Write},
     iter, str, thread,
@@ -65,7 +68,7 @@ const MARIADB: &Profile = &Profile {
     blob_type: "BLOB",
 };
 
-/// Verify writer panics if too large elements are inserted into a binary column of ColumnarRowSet
+/// Verify writer panics if too large elements are inserted into a binary column of ColumnarBuffer
 /// buffer.
 #[test]
 #[should_panic]
@@ -75,14 +78,14 @@ fn insert_too_large_element_in_bin_column() {
         kind: BufferKind::Binary { length: 1 },
         nullable: true,
     };
-    let mut buffer = ColumnarRowSet::new(10, iter::once(desc));
+    let mut buffer = buffer_from_description(10, iter::once(desc));
     buffer.set_num_rows(1);
     if let AnyColumnViewMut::Binary(mut col) = buffer.column_mut(0) {
         col.write(iter::once(Some(&b"too large input."[..])))
     }
 }
 
-/// Verify writer panics if too large elements are inserted into a text column of ColumnarRowSet
+/// Verify writer panics if too large elements are inserted into a text column of `ColumnarBuffer`
 /// buffer.
 #[test]
 #[should_panic]
@@ -92,7 +95,7 @@ fn insert_too_large_element_in_text_column() {
         kind: BufferKind::Text { max_str_len: 1 },
         nullable: true,
     };
-    let mut buffer = ColumnarRowSet::new(10, iter::once(desc));
+    let mut buffer = buffer_from_description(10, iter::once(desc));
     buffer.set_num_rows(1);
     if let AnyColumnViewMut::Text(mut col) = buffer.column_mut(0) {
         col.write(iter::once(Some(&b"too large input."[..])))
@@ -357,6 +360,28 @@ fn bind_char_to_wchar(profile: &Profile) {
     assert_eq!(Some(U16String::from_str("Hello").as_ustr()), buf.ustr_at(0));
 }
 
+/// Bind a BIT column to a Bit buffer.
+#[test_case(MSSQL; "Microsoft SQL Server")]
+#[test_case(MARIADB; "Maria DB")]
+#[test_case(SQLITE_3; "SQLite 3")]
+fn bind_bit(profile: &Profile) {
+    let table_name = "BindBit";
+
+    let conn = profile.setup_empty_table(table_name, &["BIT"]).unwrap();
+    let insert_sql = format!("INSERT INTO {} (a) VALUES (0),(1);", table_name);
+    conn.execute(&insert_sql, ()).unwrap();
+
+    let sql = format!("SELECT a FROM {};", table_name);
+    let cursor = conn.execute(&sql, ()).unwrap().unwrap();
+    let mut buf = SingleColumnRowSetBuffer::<Vec<Bit>>::new(3);
+    let mut row_set_cursor = cursor.bind_buffer(&mut buf).unwrap();
+    row_set_cursor.fetch().unwrap();
+    drop(row_set_cursor);
+
+    assert!(!buf.get()[0].as_bool());
+    assert!(buf.get()[1].as_bool());
+}
+
 /// Binds a buffer which is too short to a fixed sized character type. This provokes an indicator of
 /// `NO_TOTAL` on MSSQL.
 #[test_case(MSSQL; "Microsoft SQL Server")]
@@ -425,6 +450,28 @@ fn bind_varchar_to_wchar(profile: &Profile) {
     );
 }
 
+/// utf16 to utf8 conversion with one character those utf-16 representation is smaller than its utf8
+/// representation
+#[test_case(MSSQL; "Microsoft SQL Server")]
+#[test_case(MARIADB; "Maria DB")]
+// #[test_case(SQLITE_3; "SQLite 3")] //Doesn't work on Linux
+#[cfg(not(target_os = "windows"))] // Windows does not use UTF-8 locale by default
+fn nvarchar_to_text(profile: &Profile) {
+    let table_name = "NvarcharToText";
+    let conn = profile
+        .setup_empty_table(table_name, &["NVARCHAR(1)"])
+        .unwrap();
+    // Trade mark sign (`™`) is longer in utf-8 (3 Bytes) than in utf-16 (2 Bytes).
+    let insert_sql = format!("INSERT INTO {} (a) VALUES ('™');", table_name);
+    conn.execute(&insert_sql, ()).unwrap();
+
+    let sql = format!("SELECT a FROM {};", table_name);
+    let cursor = conn.execute(&sql, ()).unwrap().unwrap();
+    let text = cursor_to_string(cursor);
+
+    assert_eq!("™", text);
+}
+
 #[test_case(MSSQL; "Microsoft SQL Server")]
 #[test_case(MARIADB; "Maria DB")]
 #[test_case(SQLITE_3; "SQLite 3")]
@@ -479,7 +526,7 @@ fn columnar_fetch_varbinary(profile: &Profile) {
         kind: buffer_kind,
         nullable: true,
     };
-    let row_set_buffer = ColumnarRowSet::new(10, iter::once(buffer_desc));
+    let row_set_buffer = odbc_api::buffers::buffer_from_description(10, iter::once(buffer_desc));
     let mut cursor = cursor.bind_buffer(row_set_buffer).unwrap();
     let batch = cursor.fetch().unwrap().unwrap();
     let col_view = batch.column(0);
@@ -525,7 +572,7 @@ fn columnar_fetch_binary(profile: &Profile) {
         kind: buffer_kind,
         nullable: true,
     };
-    let row_set_buffer = ColumnarRowSet::new(10, iter::once(buffer_desc));
+    let row_set_buffer = buffer_from_description(10, iter::once(buffer_desc));
     let mut cursor = cursor.bind_buffer(row_set_buffer).unwrap();
     let batch = cursor.fetch().unwrap().unwrap();
     let col_view = batch.column(0);
@@ -576,7 +623,7 @@ fn columnar_fetch_timestamp(profile: &Profile) {
         kind: buffer_kind,
         nullable: true,
     };
-    let row_set_buffer = ColumnarRowSet::new(10, iter::once(buffer_desc));
+    let row_set_buffer = buffer_from_description(10, iter::once(buffer_desc));
     let mut cursor = cursor.bind_buffer(row_set_buffer).unwrap();
     let batch = cursor.fetch().unwrap().unwrap();
     let col_view = batch.column(0);
@@ -641,7 +688,7 @@ fn columnar_insert_timestamp(profile: &Profile) {
         kind: BufferKind::Timestamp,
         nullable: true,
     };
-    let mut buffer = ColumnarRowSet::new(10, iter::once(desc));
+    let mut buffer = buffer_from_description(10, iter::once(desc));
 
     // Input values to insert. Note that the last element has > 5 chars and is going to trigger a
     // reallocation of the underlying buffer.
@@ -704,7 +751,7 @@ fn columnar_insert_timestamp_ms(profile: &Profile) {
         kind: BufferKind::Timestamp,
         nullable: true,
     };
-    let mut buffer = ColumnarRowSet::new(10, iter::once(desc));
+    let mut buffer = buffer_from_description(10, iter::once(desc));
 
     // Input values to insert. Note that the last element has > 5 chars and is going to trigger a
     // reallocation of the underlying buffer.
@@ -769,7 +816,7 @@ fn columnar_insert_varbinary(profile: &Profile) {
         kind: BufferKind::Binary { length: 5 },
         nullable: true,
     };
-    let mut buffer = ColumnarRowSet::new(4, iter::once(desc));
+    let mut buffer = buffer_from_description(4, iter::once(desc));
 
     // Input values to insert. Note that the last element has > 5 chars and is going to trigger a
     // reallocation of the underlying buffer.
@@ -824,7 +871,7 @@ fn columnar_insert_varchar(profile: &Profile) {
         kind: BufferKind::Text { max_str_len: 5 },
         nullable: true,
     };
-    let mut buffer = ColumnarRowSet::new(4, iter::once(desc));
+    let mut buffer = buffer_from_description(4, iter::once(desc));
 
     // Input values to insert. Note that the last element has > 5 chars and is going to trigger a
     // reallocation of the underlying buffer.
@@ -862,6 +909,47 @@ fn columnar_insert_varchar(profile: &Profile) {
     assert_eq!(expected, actual);
 }
 
+#[test_case(MSSQL; "Microsoft SQL Server")]
+#[test_case(MARIADB; "Maria DB")]
+#[test_case(SQLITE_3; "SQLite 3")]
+fn columnar_insert_text_as_sql_integer(profile: &Profile) {
+    let table_name = "ColumnarInsertTextAsSqlInteger";
+    // Setup
+    let conn = profile.setup_empty_table(table_name, &["INTEGER"]).unwrap();
+
+    let column_buffer = WithDataType {
+        value: TextColumn::new(4, 5),
+        data_type: DataType::Integer,
+    };
+
+    // Fill buffer with values
+    let mut buffer = ColumnarBuffer::new(vec![(1, column_buffer)]);
+
+    // Input values to insert.
+    let input = [Some(&b"1"[..]), Some(&b"2"[..]), None, Some(&b"4"[..])];
+
+    buffer.set_num_rows(input.len());
+    let mut writer = buffer.column_mut(0);
+
+    writer.write(input.iter().copied());
+
+    // Bind buffer and insert values.
+    conn.execute(
+        &format!("INSERT INTO {} (a) VALUES (?)", table_name),
+        &buffer,
+    )
+    .unwrap();
+
+    // Query values and compare with expectation
+    let cursor = conn
+        .execute(&format!("SELECT a FROM {} ORDER BY Id", table_name), ())
+        .unwrap()
+        .unwrap();
+    let actual = cursor_to_string(cursor);
+    let expected = "1\n2\nNULL\n4";
+    assert_eq!(expected, actual);
+}
+
 #[test_case(MSSQL; "Microsoft SQL Server")]
 #[test_case(MARIADB; "Maria DB")]
 #[test_case(SQLITE_3; "SQLite 3")]
@@ -889,7 +977,7 @@ fn adaptive_columnar_insert_varchar(profile: &Profile) {
         Some(&b"Hello, World!"[..]),
     ];
 
-    let mut buffer = ColumnarRowSet::new(input.len(), iter::once(desc));
+    let mut buffer = buffer_from_description(input.len(), iter::once(desc));
 
     buffer.set_num_rows(input.len());
     if let AnyColumnViewMut::Text(mut writer) = buffer.column_mut(0) {
@@ -943,7 +1031,7 @@ fn adaptive_columnar_insert_varbin(profile: &Profile) {
         Some(&b"Hello, World!"[..]),
     ];
 
-    let mut buffer = ColumnarRowSet::new(input.len(), iter::once(desc));
+    let mut buffer = buffer_from_description(input.len(), iter::once(desc));
 
     buffer.set_num_rows(input.len());
     if let AnyColumnViewMut::Binary(mut writer) = buffer.column_mut(0) {
@@ -988,7 +1076,7 @@ fn columnar_insert_wide_varchar(profile: &Profile) {
         kind: BufferKind::WText { max_str_len: 5 },
         nullable: true,
     };
-    let mut buffer = ColumnarRowSet::new(10, iter::once(desc));
+    let mut buffer = buffer_from_description(10, iter::once(desc));
 
     // Input values to insert. Note that the last element has > 5 chars and is going to trigger a
     // reallocation of the underlying buffer.
@@ -1029,20 +1117,25 @@ fn columnar_insert_wide_varchar(profile: &Profile) {
     assert_eq!(expected, actual);
 }
 
-#[test]
-fn bind_integer_parameter() {
-    let conn = ENV
-        .connect_with_connection_string(MSSQL_CONNECTION)
+#[test_case(MSSQL; "Microsoft SQL Server")]
+#[test_case(MARIADB; "Maria DB")]
+#[test_case(SQLITE_3; "SQLite 3")]
+fn bind_integer_parameter(profile: &Profile) {
+    let table_name = "BindIntegerParam";
+    let conn = profile
+        .setup_empty_table(table_name, &["INTEGER", "INTEGER"])
         .unwrap();
-    let sql = "SELECT title FROM Movies where year=?;";
-    let cursor = conn.execute(sql, &1968).unwrap().unwrap();
-    let mut buffer = TextRowSet::for_cursor(1, &cursor, None).unwrap();
-    let mut cursor = cursor.bind_buffer(&mut buffer).unwrap();
+    let insert = format!("INSERT INTO {} (a,b) VALUES (1,1), (2,2);", table_name);
+    conn.execute(&insert, ()).unwrap();
 
-    let batch = cursor.fetch().unwrap().unwrap();
-    let title = batch.at_as_str(0, 0).unwrap().unwrap();
+    let sql = format!("SELECT a FROM {} where b=?;", table_name);
+    let cursor = conn.execute(&sql, &1).unwrap().unwrap();
+    let actual = cursor_to_string(cursor);
+    assert_eq!("1", actual);
 
-    assert_eq!("2001: A Space Odyssey", title);
+    let cursor = conn.execute(&sql, &2).unwrap().unwrap();
+    let actual = cursor_to_string(cursor);
+    assert_eq!("2", actual);
 }
 
 /// Learning test. Insert a string ending with \0. Not a terminating zero, but the payload ending
@@ -1171,58 +1264,50 @@ fn preallocation_soundness(profile: &Profile) {
     }
 }
 
-#[test]
-fn integer_parameter_as_string() {
-    let conn = ENV
-        .connect_with_connection_string(MSSQL_CONNECTION)
+#[test_case(MSSQL; "Microsoft SQL Server")]
+#[test_case(MARIADB; "Maria DB")]
+#[test_case(SQLITE_3; "SQLite 3")]
+fn integer_parameter_as_string(profile: &Profile) {
+    let table_name = "IntegerParameterAsString";
+    let conn = profile
+        .setup_empty_table(table_name, &["INTEGER", "INTEGER"])
         .unwrap();
-    let sql = "SELECT title FROM Movies where year=?;";
-    let cursor = conn
-        .execute(sql, &"1968".into_parameter())
-        .unwrap()
-        .unwrap();
-    let mut buffer = TextRowSet::for_cursor(1, &cursor, None).unwrap();
-    let mut cursor = cursor.bind_buffer(&mut buffer).unwrap();
+    let insert = format!("INSERT INTO {} (a,b) VALUES (1,1), (2,2);", table_name);
+    conn.execute(&insert, ()).unwrap();
 
-    let batch = cursor.fetch().unwrap().unwrap();
-    let title = batch.at_as_str(0, 0).unwrap().unwrap();
+    let sql = format!("SELECT a FROM {} where b=?;", table_name);
+    let cursor = conn.execute(&sql, &"2".into_parameter()).unwrap().unwrap();
+    let actual = cursor_to_string(cursor);
 
-    assert_eq!("2001: A Space Odyssey", title);
+    assert_eq!("2", actual);
 }
 
-#[test]
-fn parameter_option_integer_some() {
-    let conn = ENV
-        .connect_with_connection_string(MSSQL_CONNECTION)
+#[test_case(MSSQL; "Microsoft SQL Server")]
+#[test_case(MARIADB; "Maria DB")]
+#[test_case(SQLITE_3; "SQLite 3")]
+fn bind_optional_integer_parameter(profile: &Profile) {
+    let table_name = "ParameterOptionIntegerSome";
+    let conn = profile
+        .setup_empty_table(table_name, &["INTEGER", "INTEGER"])
         .unwrap();
-    let sql = "SELECT title FROM Movies where year=?;";
+    let insert = format!("INSERT INTO {} (a,b) VALUES (1,1), (2,2);", table_name);
+    conn.execute(&insert, ()).unwrap();
+
+    let sql = format!("SELECT a FROM {} where b=?;", table_name);
+
     let cursor = conn
-        .execute(sql, &Some(1968).into_parameter())
+        .execute(&sql, &Some(2).into_parameter())
         .unwrap()
         .unwrap();
-    let mut buffer = TextRowSet::for_cursor(1, &cursor, None).unwrap();
-    let mut cursor = cursor.bind_buffer(&mut buffer).unwrap();
+    let actual = cursor_to_string(cursor);
+    assert_eq!("2", actual);
 
-    let batch = cursor.fetch().unwrap().unwrap();
-    let title = batch.at_as_str(0, 0).unwrap().unwrap();
-
-    assert_eq!("2001: A Space Odyssey", title);
-}
-
-#[test]
-fn parameter_option_integer_none() {
-    let conn = ENV
-        .connect_with_connection_string(MSSQL_CONNECTION)
-        .unwrap();
-    let sql = "SELECT title FROM Movies where year=?;";
     let cursor = conn
-        .execute(sql, &None::<i32>.into_parameter())
+        .execute(&sql, &None::<i32>.into_parameter())
         .unwrap()
         .unwrap();
-    let mut buffer = TextRowSet::for_cursor(1, &cursor, None).unwrap();
-    let mut cursor = cursor.bind_buffer(&mut buffer).unwrap();
-
-    assert!(cursor.fetch().unwrap().is_none());
+    let actual = cursor_to_string(cursor);
+    assert_eq!("", actual);
 }
 
 #[test_case(MSSQL; "Microsoft SQL Server")]
@@ -1270,7 +1355,7 @@ fn wchar(profile: &Profile) {
         nullable: false,
         kind: BufferKind::WText { max_str_len: 1 },
     };
-    let row_set_buffer = ColumnarRowSet::new(2, iter::once(desc));
+    let row_set_buffer = buffer_from_description(2, iter::once(desc));
     let mut row_set_cursor = cursor.bind_buffer(row_set_buffer).unwrap();
     let batch = row_set_cursor.fetch().unwrap().unwrap();
     let col = batch.column(0);
@@ -1304,20 +1389,23 @@ fn wchar_as_char() {
     assert_eq!("A\nÜ", output);
 }
 
-#[test]
-fn two_parameters_in_tuple() {
-    let conn = ENV
-        .connect_with_connection_string(MSSQL_CONNECTION)
-        .unwrap();
-    let sql = "SELECT title FROM Movies where ? < year AND year < ?;";
-    let cursor = conn.execute(sql, (&1960, &1970)).unwrap().unwrap();
-    let mut buffer = TextRowSet::for_cursor(1, &cursor, None).unwrap();
-    let mut cursor = cursor.bind_buffer(&mut buffer).unwrap();
+#[test_case(MSSQL; "Microsoft SQL Server")]
+#[test_case(MARIADB; "Maria DB")]
+#[test_case(SQLITE_3; "SQLite 3")]
+fn two_parameters_in_tuple(profile: &Profile) {
+    let table_name = "TwoParmetersInTuple";
+    let conn = profile.setup_empty_table(table_name, &["INTEGER"]).unwrap();
+    let insert = format!("INSERT INTO {} (a) VALUES (1), (2), (3), (4);", table_name);
+    conn.execute(&insert, ()).unwrap();
 
-    let batch = cursor.fetch().unwrap().unwrap();
-    let title = batch.at_as_str(0, 0).unwrap().unwrap();
+    let sql = format!(
+        "SELECT a FROM {} where ? < a AND a < ? ORDER BY id;",
+        table_name
+    );
 
-    assert_eq!("2001: A Space Odyssey", title);
+    let cursor = conn.execute(&sql, (&1, &4)).unwrap().unwrap();
+    let actual = cursor_to_string(cursor);
+    assert_eq!("2\n3", actual);
 }
 
 #[test_case(MSSQL; "Microsoft SQL Server")]
@@ -1406,7 +1494,7 @@ fn bulk_insert_with_text_buffer(profile: &Profile) {
     let mut prepared = conn
         .prepare("INSERT INTO BulkInsertWithTextBuffer (a) Values (?)")
         .unwrap();
-    let mut params = TextRowSet::new(5, [50].iter().copied());
+    let mut params = TextRowSet::from_max_str_lens(5, [50].iter().copied());
     params.append(["England"].iter().map(|s| Some(s.as_bytes())));
     params.append(["France"].iter().map(|s| Some(s.as_bytes())));
     params.append(["Germany"].iter().map(|s| Some(s.as_bytes())));
@@ -1449,7 +1537,7 @@ fn bulk_insert_with_columnar_buffer(profile: &Profile) {
     ]
     .iter()
     .copied();
-    let mut params = ColumnarRowSet::new(5, description);
+    let mut params = buffer_from_description(5, description);
     params.set_num_rows(3);
     let mut view_mut = params.column_mut(0);
     // Fill first column with text
@@ -1648,7 +1736,7 @@ fn read_into_columnar_buffer(profile: &Profile) {
             kind: BufferKind::Text { max_str_len: 20 },
         },
     ];
-    let buffer = ColumnarRowSet::new(20, buffer_description.iter().copied());
+    let buffer = buffer_from_description(20, buffer_description.iter().copied());
     let mut cursor = cursor.bind_buffer(buffer).unwrap();
     // Assert existence of first batch
     let batch = cursor.fetch().unwrap().unwrap();
@@ -1685,7 +1773,7 @@ fn ignore_output_column(profile: &Profile) {
         kind: BufferKind::I32,
         nullable: true,
     };
-    let buffer = ColumnarRowSet::with_column_indices(20, [(1, bd), (3, bd)].iter().copied());
+    let buffer = buffer_from_description_and_indices(20, [(1, bd), (3, bd)].iter().copied());
     let mut cursor = cursor.bind_buffer(buffer).unwrap();
 
     // Assert that there is no batch.
@@ -1718,8 +1806,11 @@ fn output_parameter(profile: &Profile) {
     let mut ret = Nullable::<i32>::null();
     let mut param = Nullable::<i32>::new(7);
 
-    conn.execute("{? = call TestOutputParam(?)}", (Out(&mut ret), &mut param))
-        .unwrap();
+    conn.execute(
+        "{? = call TestOutputParam(?)}",
+        (Out(&mut ret), InOut(&mut param)),
+    )
+    .unwrap();
 
     // See magic numbers hardcoded in setup.sql
     assert_eq!(Some(99), ret.into_opt());
@@ -2423,10 +2514,12 @@ fn send_long_data_binary_file(profile: &Profile) {
 
 /// Demonstrate how to strip abstractions and access raw functionality as exposed by `odbc-sys`.
 #[test_case(MSSQL; "Microsoft SQL Server")]
-// #[test_case(MARIADB; "Maria DB")]
-// #[test_case(SQLITE_3; "SQLite 3")]
+#[test_case(MARIADB; "Maria DB")]
+#[test_case(SQLITE_3; "SQLite 3")]
 fn escape_hatch(profile: &Profile) {
-    let conn = profile.connection().unwrap();
+    let table_name = "EscapeHatch";
+    let conn = profile.setup_empty_table(table_name, &["INTEGER"]).unwrap();
+
     let preallocated = conn.preallocate().unwrap();
     let mut statement = preallocated.into_statement();
 
@@ -2434,7 +2527,7 @@ fn escape_hatch(profile: &Profile) {
 
     unsafe {
         // TableName does not exist, but we won't execute the query anyway
-        let select = U16String::from_str("SELECT * FROM Movies");
+        let select = U16String::from_str("SELECT * FROM EscapeHatch");
         let ret = sys::SQLPrepareW(
             statement.as_sys(),
             select.as_ptr(),
@@ -2562,20 +2655,23 @@ fn current_catalog(profile: &Profile, expected_catalog: &str) {
     assert_eq!(conn.current_catalog().unwrap(), expected_catalog);
 }
 
-#[test]
-fn columns_query() {
-    let conn = ENV
-        .connect_with_connection_string(MSSQL.connection_string)
+#[test_case(MSSQL; "Microsoft SQL Server")]
+#[test_case(MARIADB; "Maria DB")]
+#[test_case(SQLITE_3; "SQLite 3")]
+fn columns_query(profile: &Profile) {
+    let table_name = "ColumnsQuery";
+    let conn = profile
+        .setup_empty_table(table_name, &["VARCHAR(10)"])
         .unwrap();
 
-    let row_set_buffer = ColumnarRowSet::new(
+    let row_set_buffer = buffer_from_description(
         2,
         conn.columns_buffer_description(255, 255, 255)
             .unwrap()
             .into_iter(),
     );
     let columns = conn
-        .columns(&conn.current_catalog().unwrap(), "dbo", "Movies", "")
+        .columns(&conn.current_catalog().unwrap(), "dbo", table_name, "a")
         .unwrap();
 
     let mut cursor = columns.bind_buffer(row_set_buffer).unwrap();
@@ -2590,11 +2686,11 @@ fn columns_query() {
     const COLUMN_SIZE_INDEX: usize = 6;
     let column_sizes = i32::as_nullable_slice(batch.column(COLUMN_SIZE_INDEX)).unwrap();
 
-    let has_title_col_with_expected_size = column_names.zip(column_sizes).any(|(name, size)| {
-        str::from_utf8(name.unwrap()).unwrap() == "title" && *size.unwrap() == 255
-    });
+    let column_has_name_a_and_size_10 = column_names
+        .zip(column_sizes)
+        .any(|(name, size)| str::from_utf8(name.unwrap()).unwrap() == "a" && *size.unwrap() == 10);
 
-    assert!(has_title_col_with_expected_size);
+    assert!(column_has_name_a_and_size_10);
 }
 
 /// Demonstrating how to fill a vector of rows using this crate.
@@ -2625,7 +2721,7 @@ fn fill_vec_of_rows(profile: &Profile) {
         },
     ];
 
-    let buffer = ColumnarRowSet::new(1, buf_desc.iter().copied());
+    let buffer = buffer_from_description(1, buf_desc.iter().copied());
     let mut cursor = cursor.bind_buffer(buffer).unwrap();
 
     let mut actual = Vec::new();
@@ -2694,7 +2790,7 @@ fn row_array_size_66536(profile: &Profile) {
     let conn = profile.setup_empty_table(table_name, &["BIT"]).unwrap();
     let sql = format!("SELECT a FROM {}", table_name);
     let cursor = conn.execute(&sql, ()).unwrap().unwrap();
-    let row_set_buffer = ColumnarRowSet::new(
+    let row_set_buffer = buffer_from_description(
         u16::MAX as usize + 1,
         iter::once(BufferDescription {
             kind: BufferKind::Bit,
@@ -2707,8 +2803,10 @@ fn row_array_size_66536(profile: &Profile) {
 #[test_case(MSSQL; "Microsoft SQL Server")]
 #[test_case(MARIADB; "Maria DB")]
 #[test_case(SQLITE_3; "SQLite 3")]
-fn execute_query_twice_with_different_args_by_modifying_bound_param_buffer(profile: &Profile) {
-    let table_name = "ExecuteQueryTwiceWithDifferentArgsByModifyingBoundParamBuffer";
+fn execute_query_twice_with_different_args_by_modifying_bound_param_buffer_on_stack(
+    profile: &Profile,
+) {
+    let table_name = "ModifyingBoundParamBufferOnStack";
     let conn = profile
         .setup_empty_table(table_name, &["INTEGER", "INTEGER"])
         .unwrap();
@@ -2718,28 +2816,54 @@ fn execute_query_twice_with_different_args_by_modifying_bound_param_buffer(profi
     let query = format!("SELECT a FROM {} WHERE b=?;", table_name);
     let prepared = conn.prepare(&query).unwrap();
 
-    // Prepared statement has not yet any abstraction to keep parameters bound between execution.
-    let mut stmt = prepared.into_statement();
-
     // Stack allocated parameter. Used for both query executions.
     let mut b = 1;
 
-    let cursor = unsafe {
-        stmt.bind_input_parameter(1, &b);
-        stmt.execute();
-        CursorImpl::new(&mut stmt)
-    };
+    let mut prebound = prepared.bind_parameters(&mut b).unwrap();
+
+    let cursor = prebound.execute().unwrap().unwrap();
 
     assert_eq!("1", cursor_to_string(cursor));
 
     // Execute a second time, with a different argument, but without rebinding the parameter buffer.
-    b = 2;
+    // This assignment is indeed used, but the Rust tooling doesn't know about the pointer to `b`
+    // keeping track of it.
+    *prebound.params_mut() = 2;
 
-    let cursor = unsafe {
-        stmt.bind_input_parameter(1, &b);
-        stmt.execute();
-        CursorImpl::new(&mut stmt)
-    };
+    let cursor = prebound.execute().unwrap().unwrap();
+
+    assert_eq!("2", cursor_to_string(cursor));
+}
+
+#[test_case(MSSQL; "Microsoft SQL Server")]
+#[test_case(MARIADB; "Maria DB")]
+#[test_case(SQLITE_3; "SQLite 3")]
+fn modifying_bound_param_buffer_on_heap(profile: &Profile) {
+    let table_name = "ModifyingBoundParamBufferOnHeap";
+    let conn = profile
+        .setup_empty_table(table_name, &["INTEGER", "INTEGER"])
+        .unwrap();
+    let insert = format!("INSERT INTO {} (a,b) VALUES (1,1), (2,2);", table_name);
+    conn.execute(&insert, ()).unwrap();
+
+    let query = format!("SELECT a FROM {} WHERE b=?;", table_name);
+    let prepared = conn.prepare(&query).unwrap();
+
+    // Heap allocated parameter. Used for both query executions.
+    let b = Box::new(1);
+
+    let mut prebound = prepared.bind_parameters(b).unwrap();
+
+    let cursor = prebound.execute().unwrap().unwrap();
+
+    assert_eq!("1", cursor_to_string(cursor));
+
+    // Execute a second time, with a different argument, but without rebinding the parameter buffer.
+    // This assignment is indeed used, but the Rust tooling doesn't know about the pointer to `b`
+    // keeping track of it.
+    *prebound.params_mut() = 2;
+
+    let cursor = prebound.execute().unwrap().unwrap();
 
     assert_eq!("2", cursor_to_string(cursor));
 }
@@ -2763,7 +2887,7 @@ fn many_diagnostic_messages() {
     let batch_size = 2 << 15;
 
     // Fill each row in the buffer with two letters.
-    let mut buffer = TextRowSet::new(batch_size, iter::once(2));
+    let mut buffer = TextRowSet::from_max_str_lens(batch_size, iter::once(2));
 
     for _ in 0..batch_size {
         buffer.append([Some(&b"ab"[..])].iter().cloned());
@@ -2773,7 +2897,7 @@ fn many_diagnostic_messages() {
     conn.execute(&insert_sql, &buffer).unwrap();
 
     let query_sql = format!("SELECT a FROM {}", table_name);
-    buffer = TextRowSet::new(batch_size, iter::once(1));
+    buffer = TextRowSet::from_max_str_lens(batch_size, iter::once(1));
     let cursor = conn.execute(&query_sql, ()).unwrap().unwrap();
     let mut row_set_cursor = cursor.bind_buffer(buffer).unwrap();
 
